#!/usr/bin/env node
var spawn = require("child-process-promise").spawn;
var co = require("co");
var prompt = require("co-prompt");
var CryptoJS = require("crypto-js");
var formurlencoded = require("form-urlencoded");
var Client = require("node-rest-client").Client;
var md5crypt = require("./libs/md5crypt");
var fs = require("fs");

var client = new Client();

class Ws {
  constructor() {
    this.limit = 25;
    this.offset = 0;
    this.files = [];
  }

  async validateLoginSession() {
    // /api/user_data/
    const res = await this.call("user_data", {});
    //console.log("WS Data",res.response.vip[0]);
    if (res.response.status[0] === "OK" && +res.response.vip[0] === 1) {
      return true;
    } else {
      return false;
    }
  }

  async login(username, password) {
    const salt = await this.getSalt(username);
    const token = await this.getToken(username, password, salt);

    this.token = token;

    if (!token) return "Login Failed!";

    return "Login successed.";
  }

  async getSalt(username) {
    const res = await this.call("salt", { username_or_email: username });
    if (res.response.status[0] == "OK" && res.response.salt != "reset") {
      return res.response.salt[0].trim();
    } else {
      console.log(res);
    }
  }

  async getToken(username, password, salt) {
    if (!salt) {
      salt = await this.getSalt(username);
    }

    const sha1 = CryptoJS.SHA1(md5crypt.md5crypt(password, salt)).toString();
    const digest = CryptoJS.MD5(username + ":Webshare:" + password).toString();

    var data = {
      username_or_email: username,
      password: sha1,
      digest: digest,
      keep_logged_in: 1,
    };

    const resOut = await this.call("login", data);
    if (resOut.response.status[0] == "OK") {
      return resOut.response.token[0];
    }
  }

  list(list) {
    list.forEach((element, index) => {
      console.log("(" + (index + 1) + ")", element.name[0]);
    });
  }

  async getLink(ident) {
    const res = await this.call("file_link", { ident: ident });
    if (res.response.status[0] === "OK") {
      return res.response.link[0];
    } else {
      return res.response.status[0];
    }
  }

  async call(method, data) {
    if (this.token) data.wst = this.token;
    var args = {
      headers: {
        Accept: "application/xml; charset=UTF-8",
        "Content-Type": "application/x-www-form-urlencoded",
      },
      dataType: "text",
      data: formurlencoded(data),
    };
    return new Promise((resolve, reject) => {
      client.post(
        "http://webshare.cz/api/" + method + "/",
        args,
        function (data, response) {
          resolve(data);
        }
      );
    });
  }

  printHelp() {
    console.log("Its unstable as hell try to read readme");
    return;
  }
}

module.exports = { Ws };

// node
//const {Ws} = require('./index.js')
//const webshare = new Ws();
//await webshare.getToken("username","password");
//await webshare.getLink("K52KYq31jl");
